@@ -445,6 +445,16 @@ namespace muGrid {
     return field_names;
   }
 
+  /* ---------------------------------------------------------------------- */
+  std::vector<std::string>
+  FieldCollection::list_state_field_unique_prefixes() const {
+    std::vector<std::string> unique_prefixes;
+    for (const auto & map : this->state_fields) {
+      unique_prefixes.push_back(std::get<0>(map));
+    }
+    return unique_prefixes;
+  }
+
   /* ---------------------------------------------------------------------- */
   void FieldCollection::preregister_map(
       std::shared_ptr<std::function<void()>> & call_back) {
@@ -475,6 +485,10 @@ namespace muGrid {
   FieldCollection::register_field<Uint>(const std::string &, const Index_t &,
                                         const std::string &, const Unit &);
 
+  template TypedField<Index_t> &
+  FieldCollection::register_field<Index_t>(const std::string &, const Index_t &,
+                                           const std::string &, const Unit &);
+
   /* ---------------------------------------------------------------------- */
   FieldCollection::PixelIndexIterable::PixelIndexIterable(
       const FieldCollection & collection)
@@ -581,4 +595,22 @@ namespace muGrid {
   FieldCollection::IndexIterable::iterator::iterator(
       const PixelIndexIterator_t & pixel_index_iterator, const size_t & stride)
       : stride{stride}, pixel_index_iterator{pixel_index_iterator} {}
+
+  /* ---------------------------------------------------------------------- */
+  std::ostream &
+  operator<<(std::ostream & os,
+             const muGrid::FieldCollection::ValidityDomain & value) {
+    const char* s = 0;
+    switch (value) {
+    case (muGrid::FieldCollection::ValidityDomain::Global):
+      s = "ValidityDomain::Global";
+      break;
+    case(muGrid::FieldCollection::ValidityDomain::Local):
+      s = "ValidityDomain::Local";
+      break;
+    default:
+      s = "ValidityDomain is not defined";
+    }
+    return os << s;
+  }
 }  // namespace muGrid
